# $Id: Plugin.pm 1892 2011-07-29 05:53:42Z mats $

package Plugins::squeezedVera::Plugin;

##############################################################################
# Changelog:
#
# 2008-12-31 v0.0.1 - First version, just trying to convert the HelloWorld
#                     application and get it up and running.
# 2009-01-01 v0.0.2 - Trying out how to execute a script.
# 2009-01-03 v0.0.4 - A first version that can turn on/off lights.
# 2009-01-04 v0.0.5 - Added a text "Running..." to be displayed when the
#                     script is executing.
#                     Rewrote the web interface so that several scripts can 
#                     be defined there.
# 2009-01-11 v0.0.6 - Some cleaning up.
# 2009-01-11 v0.0.7 - Added events executing scripts at power on and off.
# 2009-01-22 v0.1.0 - Changed so that settings for power on/off scripts take
#                     effect without the need to navigate into the plugin 
#                     from the squeezebox.
#                     Made it possible to enter an empty string into the 
#                     inputs on the web interface settings.
#                     Implemented a logging facility.
#                     More cleaning up.
#                     Modified so that there now are separate web settings
#                     for on and off.
#                     Added an event for play being pressed.
#                     Added an event for alarms timing out.
# 2009-04-29 v1.0.0 - Removed extensions from scripts when shown on display.
#                     Added a SqueezedVera alarm that can run scripts at
#                     times set in the SqueezeBox.
# 2009-04-30 v1.1.1 - Corrected some bugs in the SqueezedVera alarm.
#                     Added an interface to squeezedVeraScenes.
# 2009-05-07 v1.1.2 - Corrected a bug in the way alarm scripts were added and
#                     deleted through the web interface.
# 2011-02-01 v1.1.3 - Removed some debug logs. Tested on squeezebox server 
#                     7.5.3 where it seems to work.
# 2011-02-01 v1.1.4 - Added the squeezedVera level in the folder hierarcy.
# 2011-02-03 v1.1.5 - Updated some function calls in Settings.pm.
#                     Added some new debug loggings trying to figure out why
#                     power on and off scripts does not run after a server
#                     restart before the same script has been run on demand.
# 2011-02-04 v1.1.6 - Found and corrected a bug that made the script_path not
#                     being set at start up. Removed the logs added in 1.1.5.
# 2011-04-20 v1.1.7 - Kalle tested this version with Squeezebox server 7.5.4.
# 2011-04-20 v1.1.8 - Tested with Squeezebox server 7.5.5.
# 2011-07-29 v1.1.9 - Tested with Squeezebox server 7.6.0.
# 2012-01-03 v1.1.11 - Kalle reported that it works with Squeezebox server 
#                      7.7.1. Version 1.1.10 has somehow been lost on my 
#                      system, but exists as a downloadable module on the
#                      internet.
# 2012-01-03 v1.1.12 - Forgot to step the version number in 1.1.11... 
# 2012-09-06 v1.1.13 - Reported to work with Squeezebox server 7.7.2.
# 2013-10-24 v1.1.14 - Reported in the forum to work with server 7.8.0, see
#                      reply #21 in
#                      http://forum.micasaverde.com/index.php?topic=1047.15
##############################################################################
# ToDO:
#
# - Why is the connection to squeezedVeraScenes not working?
# - Align the timer running squeezedVeraScenes to whole hours.
# - Make sure we don't lose alarms at server restart. Use preferences:
#   my $prefs = preferences('plugin.squeezedvera');?
# - Implement also 12h time in alarm, with am/pm indication.
# - The alarm date and month should be set also when pressing "left" on their
#   settings menu.
# - On the Squeezebox indicate the success/failure of the script execution,
#   when running on demand scripts.
# - Add an arrow on the display indicating that right button might be pushed.
# - Release 1.x+1.y
# - Delete some of the old versions from the web
# - Execute scripts also from the Squeezcenter web GUI, see email from Bernt
#   Karasch (Home automation).
# - Trigger scenes that are defined within Vera, see the discussions at the 
#   Micasa forum: http://forum.micasaverde.com/index.php?topic=1047.0
# - Release 2.x.y
#
##############################################################################
# Installation:
#
# - In /var/lib/squeezecenter/Plugins (might be other paths on different
#   systems) unzip the file squeezedVera-x.y.z.zip
# - Restart squeezecenter, /etc/init.d/squeezecenter restart
#
##############################################################################
# Notes:
#
# - Slim functions are located here: /usr/lib/perl5/vendor_perl/5.8.8/Slim
# - To build the package go to: 
#   $ ~/svndev/repos2/squeezedVera/trunk (or rather tags/release-x.y.z)
#   $ zip -r squeezedVera-x.y.z.zip * 
#
##############################################################################
# This plugin is based on:
# - Power Switch II by Felix Mueller
# - HelloWorld tutorial plugin by Mitch Gerdisch
#
# HelloWorld is derived from code with the following copyright message:
#
# SqueezeCenter Copyright 2001-2007 Logitech.
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 2.
##############################################################################

# required bit
use strict;

# required bit
use base qw(Slim::Plugin::Base);

# This points to the Settings.pm file
use Plugins::squeezedVera::Settings;

# Might be required. At least it's not bad to include it.
use Scalar::Util qw(blessed);

# Might be required. Not sure, but again it doesn't hurt.
use Slim::Control::Request;

# MUST HAVE: This log bit is new and allows one to use nice logging
# facilities
use Slim::Utils::Log;

# MUST HAVE: magical preferences getting stuff
use Slim::Utils::Prefs;

# The strings functionality that uses the strings.txt file to present the
# correct language
use Slim::Utils::Strings qw(string);

use Cwd;

# So, get the data related to this plugin.
my $prefs = preferences('plugin.squeezedvera');

# Global variables are declared and/or set here
our @browseMenuChoices;
our @browseMenuChoicesWithBars;
our @alarmMenuChoices;
our @setAlarmMenuChoices;
our @setDateMenuChoices;
our @setMonthMenuChoices;
our @setScriptsMenuChoices;
our @scenesMenuChoices;
our @allScenesMenu;
# alarmData contains the following information:
# alarmData[0]: ON/OFF
# alarmData[1]: Alarm number
# alarmData[2]: Hour
# alarmData[3]: Minute
# alarmData[4]: Date
# alarmData[5]: Month
# alarmData[6]: Alarm ID
# alarmData[7]: Script
our @alarmData;
our $alarmOnOff;
our $pickedAlarm;
our $script_path;
our $lastAlarmId;
our $alarmTimeOnChange;
our $timerId;
our $newScene;
our $sqvsTimerId;

# Logging
# Log levels are DEBUG, INFO, WARN, ERROR, FATAL where a level will include 
# messages for all levels to the right.
# The "persist" bit of text allows the system to remember that logging level 
# between invocations of squeezecenter.
my $log = Slim::Utils::Log->addLogCategory({
	'category'     => 'plugin.squeezedvera',
#	'defaultLevel' => 'INFO',
	'defaultLevel' => 'DEBUG',
	'description'  => getDisplayName(),
});

# Debug routine
sub myDebug {
	my $msg = shift;
	my $lvl = shift;
	
	if ($lvl eq "")
	{
		$lvl = "debug";
	}
	
	$log->$lvl("*** squeezedVeraLogging *** $msg");
}

# Returns the name to display on the squeezebox
sub getDisplayName {
    return 'PLUGIN_SQUEEZED_VERA_TITLE';
}

# initPlugin is called when the server starts up and is loading the plugins.
# Used to initialize variables and the like.
sub initPlugin {
    my $class = shift;
   
    myDebug("initPlugin");
    
    # Next two things have to be done when initializing the plugin.
    $class->SUPER::initPlugin();
    # This may instantiate the web interface for the plugin.
    Plugins::squeezedVera::Settings->new;

    $lastAlarmId = 0;
    
    # Using the Input modes capability of slimserver to control the menues.
    # Check if the name field is populated in the plugin's preferences store
    # and if not, set it to a default
    if ($prefs->get('demand_scripts') eq '') {	
        $prefs->set('demand_scripts',
                    "script_1;script_2;script_3");
    }
    if ($prefs->get('pwr_on_scripts') eq '') {	
        $prefs->set('pwr_on_scripts',
                    "script_1,player_1;script_2,player_2");
    }
    if ($prefs->get('pwr_off_scripts') eq '') {	
        $prefs->set('pwr_off_scripts',
                    "script_1,player_1;script_2,player_2");
    }
    if ($prefs->get('play_scripts') eq '') {	
        $prefs->set('play_scripts',
                    "script_1,player_1;script_2,player_2");
    }
    if ($prefs->get('alarm_timeout_scripts') eq '') {	
        $prefs->set('alarm_timeout_scripts',
                    "script_1,player_1;script_2,player_2");
    }
    if ($prefs->get('alarm_scripts') eq '') {	
        $prefs->set('alarm_scripts',
                    "script_1;script_2");
    }
    if ($prefs->get('sqvs_path') eq '') {	
        $prefs->set('sqvs_path',
                    "/path/to/squeezedVeraScenes.pl");
    }
    if ($prefs->get('sqvs_timer') eq '') {	
        $prefs->set('sqvs_timer',
                    "0");
    }
    
    # Upon init of the pluging extract the path to the script folder
    $script_path = $prefs->get('script_path');
    $script_path =~ s#(.+)([^/])\z#$1$2/#;

    if ($prefs->get('script_path') eq '') {
        $prefs->set('script_path',
                    "/path/to/my/scripts");
    }

    # Install callback to get client power state changes
    Slim::Control::Request::subscribe( \&commandCallback, [['power',
                                                            'play',
                                                            'playlist',
                                                            'pause',
                                                            'client',
                                                            'stop']]);

    # This is the timer that calls the function that executes
    # squeezedVeraScenec -t with a certain interval. This timer
    # is always running, to check if the user via the web has
    # decided to run squeezedVeraScenes -t, i.e. by setting
    # the time interval to >0. Per default this runs every 5
    # minutes.
    $sqvsTimerId = Slim::Utils::Timers::setTimer
        ( 0, Time::HiRes::time() + 300,
          \&sqvsTimer );    
    
}

# setMode is called when the user navigates to the plugin on the squeezebox
# and navigates in or out of it.
sub setMode {
    my $class  = shift;
    my $client = shift;
    my $method = shift;

    my $demand_scripts = $prefs->get('demand_scripts');

    # Handle requests to exit this mode/plugin by going back to where the 
    # user was before they came here.
    if ($method eq 'pop') {
        # Pop the current mode off the mode stack and restore the previous
        # one
        Slim::Buttons::Common::popMode($client);
        return;
    }

    # Upon entering the plugin, we'll provide a menu of options.
    @browseMenuChoicesWithBars = split /;/, $demand_scripts;
    $demand_scripts =~ s/_/ /g;
    $demand_scripts =~ s/\.\w+/ /g;
    @browseMenuChoices = split /;/, $demand_scripts; 
    push @browseMenuChoices, 'PLUGIN_SQUEEZED_VERA_ALARM';
    push @browseMenuChoices, 'PLUGIN_SQUEEZED_VERA_SCENES';

    my %params = (
        'listRef'        => \@browseMenuChoices, 
        'externRef'		 => \@browseMenuChoices, 
        'stringExternRef' => 1, 
        'externRefArgs'  => 'CV',
        'header'         => 'PLUGIN_SQUEEZED_VERA_TITLE', 
        'headerAddCount' => 1, 
        'stringHeader'   => 1, 
        'callback'       => \&squeezedveraMainMenu, 
        'onChange'       => \&squeezedveraOnChange, 
        'onChangeArgs'	 => 'CVI', 
    );

    # stuff that is provided by the system to scroll through the options.
    Slim::Buttons::Common::pushMode($client, 'INPUT.List', \%params);
}

# If something should be done each time a the user scrolled up or down,
# this is where it should be done.
sub squeezedveraOnChange {
   
}

# This is the function that runs squeezedVeraScenes -t, i.e.
# makes squeezedVeraScenes check if anything has triggered.
sub sqvsTimer {

    # If on the web the timer interval is set to 0, we will not
    # execute squeezedVeraScenes -t, and will set the timer interval
    # to the default value to be able to come back and check if the
    # timer value has been changed on the web.
    my $default_timer = 300;
    
    my $sqvs_timer = $prefs->get('sqvs_timer');
    my $interval = $sqvs_timer * 60;

    if ( $interval > 0 ) {
        my $sqvs_path = $prefs->get('sqvs_path');
        my $current_dir = getcwd;
        
        chdir $sqvs_path or die "chdir $sqvs_path: $!";
        my $result = `./squeezedVeraScenes.pl -t`;
        chdir $current_dir or die "chdir $current_dir: $!";
    } else {
        # Now set the timer to the default value
        $interval = $default_timer;
    }
    
    $sqvsTimerId = Slim::Utils::Timers::setTimer
        ( 0, Time::HiRes::time() + $interval,
          \&sqvsTimer );    
    
}

sub squeezedveraAlarmMenu {
    my ($client,$exittype) = @_;
    
    $exittype = uc($exittype);

    if ($exittype eq 'LEFT') {
        Slim::Buttons::Common::popModeRight($client);

        $client->update;
    } elsif ($exittype eq 'RIGHT') {
        my $valueref = $client->modeParam('valueRef');
        if ($$valueref eq 'PLUGIN_SQUEEZED_VERA_ALL_ALARM_OFF' or
                $$valueref eq 'PLUGIN_SQUEEZED_VERA_ALL_ALARM_ON') {
            if ( $alarmOnOff eq 'ON') {
                $alarmOnOff = 'OFF';
                $alarmMenuChoices[0] = 'PLUGIN_SQUEEZED_VERA_ALL_ALARM_OFF';
                # Stop the alarm check timer
                Slim::Utils::Timers::killSpecific( $timerId );
                myDebug("Timer $timerId stopped!");
                undef $timerId;
            } else {
                $alarmOnOff = 'ON';
                $alarmMenuChoices[0] = 'PLUGIN_SQUEEZED_VERA_ALL_ALARM_ON';
                # Start the alarm check timer
                alarmCheckTimer($client, 1);
                myDebug("Timer $timerId started!");
                
            }
            $client->update;
        } elsif ( $$valueref eq 'PLUGIN_SQUEEZED_VERA_ADD_ALARM'  ) {

            my $sec; my $min; my $hour; my $mday; my $mon; my $year;
            my $wday; my $yday; my $isdst;
            
            ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
            
            if ( $hour < 10 ) {
                $hour = "0$hour";
            }
            if ( $min < 10 ) {
                $min = "0$min";
            }

            my $numAlarm;
            my $newAlarmNumber;
            my $alarmId = $lastAlarmId + 1;
            $lastAlarmId++;
            if ( $lastAlarmId == 257 ) {
                $lastAlarmId = 0;
            }
            
            if (@alarmData) {
                $numAlarm = scalar @alarmData;
                $newAlarmNumber = $numAlarm + 1;
                $alarmData[$numAlarm] =
                    "OFF,$alarmId,$hour,$min,$mday,$mon";
            } else {
                $numAlarm = 0;
                $newAlarmNumber = $numAlarm + 1;
                $alarmData[0] =
                    "OFF,$alarmId,$hour,$min,$mday,$mon";
            }

            $client->showBriefly( {
                'line'       => [ "", "Adding Alarm $newAlarmNumber" ],
                'overlay'    => [ $client->symbols('notesymbol') ],
            }, { 'duration' => 1, 'block' => 1 } );
            
            $alarmMenuChoices[$newAlarmNumber + 1] = 
                    "Alarm $newAlarmNumber (OFF)";
            $client->update;

        } else {
            if (@alarmData) {
                my $numAlarms = scalar @alarmData;
                for (my $n=0; $n<$numAlarms; $n++) {
                    my @thisAlarm = split /,/, $alarmData[$n];
                    if ( $$valueref =~ m/Alarm (\d+)/ ){
                    } 
                    if ( $1 == ($n+1) ) {
                        $pickedAlarm = $n;
                        if ( $thisAlarm[0] eq 'ON' ) {
                            $setAlarmMenuChoices[0] =
                                'PLUGIN_SQUEEZED_VERA_ALARM_ON';
                        } else {
                            $setAlarmMenuChoices[0] =
                                'PLUGIN_SQUEEZED_VERA_ALARM_OFF';
                        }
                        $setAlarmMenuChoices[1] = 
                            'PLUGIN_SQUEEZED_VERA_ALARM_SET_TIME';
                        $setAlarmMenuChoices[2] = 
                            'PLUGIN_SQUEEZED_VERA_ALARM_SET_DATE';
                        $setAlarmMenuChoices[3] = 
                            'PLUGIN_SQUEEZED_VERA_ALARM_SET_MONTH';
                        $setAlarmMenuChoices[4] =
                            'PLUGIN_SQUEEZED_VERA_ALARM_SET_SCRIPTS';
                        $setAlarmMenuChoices[5] =
                            'PLUGIN_SQUEEZED_VERA_ALARM_REMOVE';

                        my %params = (
                            'listRef'        => \@setAlarmMenuChoices, 
                            'externRef'		 => \@setAlarmMenuChoices, 
                            'stringExternRef' => 1, 
                            'externRefArgs'  => 'CV',
                            'header'         => 'PLUGIN_SQUEEZED_VERA_ALARM', 
                            'headerAddCount' => 1, 
                            'stringHeader'   => 1, 
                            'callback'       => \&squeezedveraSetAlarmMenu, 
                            'onChange'       => \&squeezedveraOnChange, 
                            'onChangeArgs'	 => 'CVI', 
                        );

                        # stuff that is provided by the system to scroll through the
                        # options.
                        Slim::Buttons::Common::pushModeLeft($client,
                                                            'INPUT.List',
                                                            \%params);
                        $client->update;
                        
                    }
                }            
            }
        }
    }
}

sub squeezedveraSetAlarmMenu {
    my ($client,$exittype) = @_;
    
    $exittype = uc($exittype);

    # To refresh the display after e.g. leaving set time.
    if ( $alarmOnOff eq 'ON') {
        $alarmMenuChoices[0] = 'PLUGIN_SQUEEZED_VERA_ALL_ALARM_ON';
    } else {
        $alarmMenuChoices[0] = 'PLUGIN_SQUEEZED_VERA_ALL_ALARM_OFF';
    }
    $alarmMenuChoices[1] = 'PLUGIN_SQUEEZED_VERA_ADD_ALARM';
    if (@alarmData) {
        my $numAlarm = scalar @alarmData;
        for (my $n=0; $n<$numAlarm; $n++) {
            my $alarmNumber = $n + 1;
            my @thisAlarm = split /,/, $alarmData[$n];
            if ( $thisAlarm[0] eq 'ON' ) {
                my @month = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
                if ( $thisAlarm[7] eq q{} ) {
                    $alarmMenuChoices[$n+2] = 
                        "Alarm $alarmNumber (".
                            "$thisAlarm[2]:$thisAlarm[3] ".
                                "$thisAlarm[4] $month[$thisAlarm[5]])";
                } else {
                    $alarmMenuChoices[$n+2] = 
                        "Alarm $alarmNumber (".
                            "$thisAlarm[2]:$thisAlarm[3] ".
                                "$thisAlarm[4] $month[$thisAlarm[5]] - $thisAlarm[7])";
                }
            } else {
                $alarmMenuChoices[$n+2] = 
                    "Alarm $alarmNumber (OFF)";
                
            }
        }
    }

    # Take care of left or right pushes
    if ($exittype eq 'LEFT') {
        Slim::Buttons::Common::popModeRight($client);
        $client->update;
    } elsif ($exittype eq 'RIGHT') {
        my $valueref = $client->modeParam('valueRef');
        if ($$valueref eq 'PLUGIN_SQUEEZED_VERA_ALARM_OFF' or
                $$valueref eq 'PLUGIN_SQUEEZED_VERA_ALARM_ON') {
            my @thisAlarm = split /,/, $alarmData[$pickedAlarm];
            my $alarmNumber = $pickedAlarm + 1;
            if ( $thisAlarm[0] eq 'ON' ) {
                $thisAlarm[0] = 'OFF';
                $setAlarmMenuChoices[0] = 'PLUGIN_SQUEEZED_VERA_ALARM_OFF';
            } else {
                $thisAlarm[0] = 'ON';
                $setAlarmMenuChoices[0] = 'PLUGIN_SQUEEZED_VERA_ALARM_ON';
            }
            $alarmData[$pickedAlarm] = join ',',
                $thisAlarm[0],
                    $alarmNumber,
                        $thisAlarm[2],
                            $thisAlarm[3],
                                $thisAlarm[4],
                                    $thisAlarm[5],
                                        $thisAlarm[6],
                                            $thisAlarm[7];
            if ( $thisAlarm[0] eq 'ON' ) {
                my @month = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
                if ( $thisAlarm[7] eq q{} ) {
                    $alarmMenuChoices[$pickedAlarm+2] = 
                        "Alarm $alarmNumber (".
                            "$thisAlarm[2]:$thisAlarm[3] ".
                                "$thisAlarm[4] $month[$thisAlarm[5]])";
                } else {
                    $alarmMenuChoices[$pickedAlarm+2] = 
                        "Alarm $alarmNumber (".
                            "$thisAlarm[2]:$thisAlarm[3] ".
                                "$thisAlarm[4] $month[$thisAlarm[5]] - $thisAlarm[7])";
                }
            } else {
                $alarmMenuChoices[$pickedAlarm+2] = 
                    "Alarm $alarmNumber (OFF)";
            }
            $client->update;
        } elsif ( $$valueref eq 'PLUGIN_SQUEEZED_VERA_ALARM_REMOVE' ) {

            my $alarmToRemove = $pickedAlarm + 1;

            Slim::Buttons::Common::popModeRight($client);
            
            $client->showBriefly( {
                'line'       => [ "", "Removing Alarm $alarmToRemove" ],
                'overlay'    => [ $client->symbols('notesymbol') ],
            },{ 'duration' => 1, 'block' => 1 } );
            
            my $numAlarm = scalar @alarmData;
            if ( $numAlarm == 1 ) {
                undef @alarmData;
                undef @alarmMenuChoices;
            } else {
                my $numAlarm = scalar @alarmData;
                my $n;
                for ($n=$pickedAlarm; $n<($numAlarm-1); $n++) {
                    $alarmData[$n] = $alarmData[$n+1];
                }
                pop @alarmData;
                undef @alarmMenuChoices;
            }
            if ( $alarmOnOff eq 'ON') {
                $alarmMenuChoices[0] = 'PLUGIN_SQUEEZED_VERA_ALL_ALARM_ON';
            } else {
                $alarmMenuChoices[0] = 'PLUGIN_SQUEEZED_VERA_ALL_ALARM_OFF';
            }
            push @alarmMenuChoices, 'PLUGIN_SQUEEZED_VERA_ADD_ALARM';
            if (@alarmData) {
                my $numAlarm = scalar @alarmData;
                for (my $n=0; $n<$numAlarm; $n++) {
                    my $alarmNumber = $n + 1;
                    my @thisAlarm = split /,/, $alarmData[$n];
                    if ( $thisAlarm[0] eq 'ON' ) {
                        my @month = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
                        if ( $thisAlarm[7] eq q{} ) {
                            $alarmMenuChoices[$n+2] = 
                                "Alarm $alarmNumber (".
                                    "$thisAlarm[2]:$thisAlarm[3] ".
                                        "$thisAlarm[4] $month[$thisAlarm[5]])";
                        } else {
                            $alarmMenuChoices[$n+2] = 
                                "Alarm $alarmNumber (".
                                    "$thisAlarm[2]:$thisAlarm[3] ".
                                        "$thisAlarm[4] $month[$thisAlarm[5]] - $thisAlarm[7])";
                        }
                    } else {
                        $alarmMenuChoices[$n+2] = 
                            "Alarm $alarmNumber (OFF)";
                    }
                }
            }
                        
        } elsif ( $$valueref eq 'PLUGIN_SQUEEZED_VERA_ALARM_SET_TIME' ) {

            my @thisAlarm = split /,/, $alarmData[$pickedAlarm];
            my $epochTime = Time::Local::timelocal(
                0,
                $thisAlarm[3],
                $thisAlarm[2],
                1,
                1,
                1);

            my $zeroTime = Time::Local::timelocal(
                0,
                0,
                0,
                1,
                1,
                1);

            my $initTime = $epochTime - $zeroTime;
            
            my %params = (
                'header'       => 'PLUGIN_SQUEEZED_VERA_ALARM_SET_TIME',
                'stringHeader' => 1,
                'cursorPos'    => 0,
                'callback'     => \&exitSetTimeHandler,
                'onChange'     => \&onChangeHandler,
                'onChangeArgs' => 'CV',
            );
            
            $params{'valueRef'} = $initTime;
            
            Slim::Buttons::Common::pushMode($client, 'INPUT.Time', \%params);

            $client->update;

        } elsif ( $$valueref eq 'PLUGIN_SQUEEZED_VERA_ALARM_SET_DATE' ) {

            my @thisAlarm = split /,/, $alarmData[$pickedAlarm];

            my $n;
            for ( $n=0; $n<31; $n++ ) {
                $setDateMenuChoices[$n] = (($thisAlarm[4] + $n - 1) % 31) + 1;
            }
            
            my %params = (
                'listRef'        => \@setDateMenuChoices, 
                'externRef'		 => \@setDateMenuChoices, 
                'stringExternRef' => 1, 
                'externRefArgs'  => 'CV',
                'header'         => 'PLUGIN_SQUEEZED_VERA_ALARM', 
                'headerAddCount' => 1, 
                'stringHeader'   => 1, 
                'callback'       => \&squeezedveraSetDateMenu, 
                'onChange'       => \&squeezedveraOnChange, 
                'onChangeArgs'	 => 'CVI', 
            );
            
            # stuff that is provided by the system to scroll through the
            # options.
            Slim::Buttons::Common::pushModeLeft($client,
                                                'INPUT.List',
                                                \%params);
            $client->update;
            
        } elsif ( $$valueref eq 'PLUGIN_SQUEEZED_VERA_ALARM_SET_MONTH' ) {

            my @thisAlarm = split /,/, $alarmData[$pickedAlarm];
            
            my @month = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
            
            my $n;
            my $monthNumber;
            for ( $n=0; $n<12; $n++ ) {
                $monthNumber = ($thisAlarm[5] + $n) % 12; 
                $setMonthMenuChoices[$n] = $month[$monthNumber];
            }
            
            my %params = (
                'listRef'        => \@setMonthMenuChoices, 
                'externRef'		 => \@setMonthMenuChoices, 
                'stringExternRef' => 1, 
                'externRefArgs'  => 'CV',
                'header'         => 'PLUGIN_SQUEEZED_VERA_ALARM', 
                'headerAddCount' => 1, 
                'stringHeader'   => 1, 
                'callback'       => \&squeezedveraSetMonthMenu, 
                'onChange'       => \&squeezedveraOnChange, 
                'onChangeArgs'	 => 'CVI', 
            );
            
            # stuff that is provided by the system to scroll through the
            # options.
            Slim::Buttons::Common::pushModeLeft($client,
                                                'INPUT.List',
                                                \%params);
            $client->update;
            
        } elsif ( $$valueref eq 'PLUGIN_SQUEEZED_VERA_ALARM_SET_SCRIPTS' ) {

            my $alarm_scripts = $prefs->get('alarm_scripts');

            my @alarmScripts  = split /;/, $alarm_scripts;

            my $numScripts = scalar @alarmScripts;
            my $n;

            if (@setScriptsMenuChoices) {
                undef @setScriptsMenuChoices;
            }

            for ($n=0; $n<$numScripts; $n++) {
                $setScriptsMenuChoices[$n] = $alarmScripts[$n];
            }
            
            my %params = (
                'listRef'        => \@setScriptsMenuChoices, 
                'externRef'		 => \@setScriptsMenuChoices, 
                'stringExternRef' => 1, 
                'externRefArgs'  => 'CV',
                'header'         => 'PLUGIN_SQUEEZED_VERA_ALARM', 
                'headerAddCount' => 1, 
                'stringHeader'   => 1, 
                'callback'       => \&squeezedveraSetScriptsMenu, 
                'onChange'       => \&squeezedveraOnChange, 
                'onChangeArgs'	 => 'CVI', 
            );
            
            # stuff that is provided by the system to scroll through the
            # options.
            Slim::Buttons::Common::pushModeLeft($client,
                                                'INPUT.List',
                                                \%params);
            $client->update;
            
        }
    }
}

sub squeezedveraSetScriptsMenu {
    my ($client,$exittype) = @_;
    
    $exittype = uc($exittype);    

    if ($exittype eq 'LEFT') {
        Slim::Buttons::Common::popModeRight($client);
        $client->update;
    } elsif ($exittype eq 'RIGHT') {
        my $pickedScript = $client->modeParam('valueRef');
        
        my @thisAlarm = split /,/, $alarmData[$pickedAlarm];
        
        $alarmData[$pickedAlarm] = join ',',
            $thisAlarm[0],
                $thisAlarm[1],
                    $thisAlarm[2],
                        $thisAlarm[3],
                            $thisAlarm[4],
                                $thisAlarm[5],
                                    $thisAlarm[6],
                                        $$pickedScript;
        
        Slim::Buttons::Common::popModeRight($client);
    } 
}

sub squeezedveraSetMonthMenu {
    my ($client,$exittype) = @_;

    $exittype = uc($exittype);    

    my %monthNumber = (
        'Jan' => 0,
        'Feb' => 1,
        'Mar' => 2,
        'Apr' => 3,
        'May' => 4,
        'Jun' => 5,
        'Jul' => 6,
        'Aug' => 7,
        'Sep' => 8,
        'Oct' => 9,
        'Nov' => 10,
        'Dec' => 11,
        );
            
    if ($exittype eq 'LEFT') {
        Slim::Buttons::Common::popModeRight($client);
        $client->update;
    } elsif ($exittype eq 'RIGHT') {
        my $pickedMonth = $client->modeParam('valueRef');

        my @thisAlarm = split /,/, $alarmData[$pickedAlarm];
        
        $alarmData[$pickedAlarm] = join ',',
            $thisAlarm[0],
                $thisAlarm[1],
                    $thisAlarm[2],
                        $thisAlarm[3],
                            $thisAlarm[4],
                                $monthNumber{$$pickedMonth},
                                    $thisAlarm[6],
                                        $thisAlarm[7];
        Slim::Buttons::Common::popModeRight($client);
        alarmCheckTimer($client, 0);
    } 
}

sub squeezedveraSetDateMenu {
    my ($client,$exittype) = @_;

    $exittype = uc($exittype);    
   
    if ($exittype eq 'LEFT') {
        Slim::Buttons::Common::popModeRight($client);
        $client->update;
    } elsif ($exittype eq 'RIGHT') {
        my $pickedDate = $client->modeParam('valueRef');

        my @thisAlarm = split /,/, $alarmData[$pickedAlarm];
        
        $alarmData[$pickedAlarm] = join ',',
            $thisAlarm[0],
                $thisAlarm[1],
                    $thisAlarm[2],
                        $thisAlarm[3],
                            $$pickedDate,
                                $thisAlarm[5],
                                    $thisAlarm[6],
                                        $thisAlarm[7];
        Slim::Buttons::Common::popModeRight($client);
        alarmCheckTimer($client, 0);
    } 
}

sub onChangeHandler {
    my $client = shift;
    my $value  = shift;
    # The changed alarmtime, updated after each button press
    $alarmTimeOnChange = $value;
}

sub exitSetTimeHandler {
    my $client = shift;

    my $sec; my $min; my $hour; my $mday; my $mon; my $year;
    my $wday; my $yday; my $isdst;
    
    ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    my $zeroTime = Time::Local::timelocal(
        0,
        0,
        0,
        $mday,
        $mon,
        $year);

    my $newAlarmTime = $zeroTime + $alarmTimeOnChange;

    ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
        localtime($newAlarmTime);

    if ( $hour < 10 ) {
        $hour = "0$hour";
    }
    if ( $min < 10 ) {
        $min = "0$min";
    }
    my @thisAlarm = split /,/, $alarmData[$pickedAlarm];
    
    $alarmData[$pickedAlarm] = join ',',
        $thisAlarm[0],
            $thisAlarm[1],
                $hour,
                    $min,
                        $thisAlarm[4],
                            $thisAlarm[5],
                                $thisAlarm[6],
                                    $thisAlarm[7];

    Slim::Buttons::Common::popModeRight($client);

    alarmCheckTimer($client, 0);
}

sub alarmCheckTimer {
    my $client  = shift;
    my $restart = shift;

    # Check if any alarm is due within less than 6 minutes.
    # If yes, then start a new timer that triggers the scripts
    # to be run.
    if ( $alarmOnOff eq 'ON' ) {

        my $alarmEpochTime;
        my $sec; my $min; my $hour; my $mday; my $mon; my $year;
        my $wday; my $yday; my $isdst;
        
        ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);

        my $numAlarms = scalar @alarmData;
        my $n;
        
        for ($n=0; $n<$numAlarms; $n++) {
                 
            my @thisAlarm = split /,/, $alarmData[$n];
            if ( $thisAlarm[0] eq 'ON' ) {

                my $now  = Time::HiRes::time();
                
                $alarmEpochTime = Time::Local::timelocal(
                    0,
                    $thisAlarm[3],
                    $thisAlarm[2],
                    $thisAlarm[4],
                    $thisAlarm[5],
                    $year);
                
                if ( $alarmEpochTime < $now ) {
                    $alarmEpochTime = Time::Local::timelocal(
                        0,
                        $thisAlarm[3],
                        $thisAlarm[2],
                        $thisAlarm[4],
                        $thisAlarm[5],
                        $year+1);
                }

                my $timeDiff = $alarmEpochTime-$now;

                # If timeDiff is less than 6 minutes, there is
                # less than 6 minutes until an alarm shall
                # trigger. In that case we fire off the short
                # timer that will trigger the alarm.
                if ( $timeDiff < 360 ) {
                    # Check if alarmData already has timer id
                    # Create timer
                    # Save timer id in alarmData
                    if ( $thisAlarm[6] ne q{} ) {
                        Slim::Utils::Timers::killSpecific( $thisAlarm[6] );
                        undef $thisAlarm[6];
                    }
                    $thisAlarm[6] = Slim::Utils::Timers::setTimer
                        ( $client, Time::HiRes::time() + $timeDiff,
                          \&alarmScripts, $thisAlarm[1] );
                    $alarmData[$n] = join ',',
                        $thisAlarm[0],
                            $thisAlarm[1],
                                $thisAlarm[2],
                                    $thisAlarm[3],
                                        $thisAlarm[4],
                                            $thisAlarm[5],
                                                $thisAlarm[6],
                                                    $thisAlarm[7];
                    
                } elsif ( $thisAlarm[6] ne q{} ) {
                    Slim::Utils::Timers::killSpecific( $thisAlarm[6] );
                    undef $thisAlarm[6];
                }
            }
        }
    }

    # Start a timer that returns to this function after 5 minutes, if not
    # already started.
    if ( $restart == 1) {
        $timerId = Slim::Utils::Timers::setTimer( $client, Time::HiRes::time() + 300,
                                                  \&alarmCheckTimer, 1 );
    }
}

sub alarmScripts {
    my $client  = shift;
    my $alarmId = shift;

    myDebug("Alarm!! Alarm: $alarmId");

    if ( $alarmOnOff eq 'ON' ) {

        my $numAlarms = scalar @alarmData;
        my $n;

        for ($n=0; $n<$numAlarms; $n++) {
            my @thisAlarm = split /,/, $alarmData[$n];
            if ( ($thisAlarm[1] == $alarmId) && ($thisAlarm[0] eq 'ON') ) {
                # Execute the script
                my $script_file = $script_path.$thisAlarm[7];
                system $script_file;
                myDebug("Executing, $thisAlarm[7]");
                

                undef $thisAlarm[6];
                
                $alarmData[$n] = join ',',
                    'OFF',
                        $thisAlarm[1],
                            $thisAlarm[2],
                                $thisAlarm[3],
                                    $thisAlarm[4],
                                        $thisAlarm[5],
                                            $thisAlarm[6],
                                                $thisAlarm[7];

                $alarmMenuChoices[$n+2] = "Alarm $thisAlarm[1] (OFF)";
                $client->update;
            }
        }
    }
}


# This subroutine is called when the user does something other than up or
# down through the main menu.
# This plugin only cares about LEFT and RIGHT presses.
# LEFT bumps you back to the main menu.
# RIGHT executes a script
sub squeezedveraMainMenu {
    my ($client,$exittype) = @_;
    
    $exittype = uc($exittype);

    # If the button is the left arrow, then just pop back up the previous
    # menu. In this case that's the main menu. But, if I had pushed down to a
    # lower menu, then it would pop up to a higher level of this plugin.
    if ($exittype eq 'LEFT') {
        
        Slim::Buttons::Common::popModeRight($client);
        
        
    } elsif ($exittype eq 'RIGHT') {
        # Do whatever should be done if the right arrow is pressed
        # Since this callback function was set from the initial menu
        # (see setmode above) it handles the top most menu options.
        # If one of those menu options has sub menues, then I  push on another
        # menu and it would have it's own callback function to handle button
        # presses.
        
        # This appears to return a reference to the current menu option being 
        # displayed. That's why you'll see the $$valueref when actually
        # checking the value below
        my $valueref = $client->modeParam('valueRef');

        my $num_of_scripts = @browseMenuChoicesWithBars;
        for (my $n = 0; $n < $num_of_scripts; $n++) {
            my $script_name = $browseMenuChoices[$n];
            if ($$valueref eq $script_name) {
                my $script_file = $script_path.$browseMenuChoicesWithBars[$n];

                $client->showBriefly( {
                    'line'       => [ "", "Running..." ],
                    'overlay'    => [ $client->symbols('notesymbol') ],
                }, { 'duration' => 1, 'block' => 1 } );

                system $script_file;
                myDebug("Executing, $browseMenuChoicesWithBars[$n]");
            }
        }

        if ($$valueref eq 'PLUGIN_SQUEEZED_VERA_ALARM') {

            if ( not defined $alarmOnOff ) {
                $alarmOnOff = 'OFF';
                $alarmMenuChoices[0] = 'PLUGIN_SQUEEZED_VERA_ALL_ALARM_OFF';
            }
            else {
                if ( $alarmOnOff eq 'ON') {
                    $alarmMenuChoices[0] = 'PLUGIN_SQUEEZED_VERA_ALL_ALARM_ON';
                } else {
                    $alarmMenuChoices[0] = 'PLUGIN_SQUEEZED_VERA_ALL_ALARM_OFF';
                }
            }
            push @alarmMenuChoices, 'PLUGIN_SQUEEZED_VERA_ADD_ALARM';
            if (@alarmData) {
                my $numAlarm = scalar @alarmData;
                for (my $n=0; $n<$numAlarm; $n++) {
                    my $alarmNumber = $n + 1;
                    my @thisAlarm = split /,/, $alarmData[$n];
                    if ( $thisAlarm[0] eq 'ON' ) {
                        my @month = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
                        if ( $thisAlarm[7] eq q{} ) {
                            $alarmMenuChoices[$n+2] = 
                                "Alarm $alarmNumber (".
                                    "$thisAlarm[2]:$thisAlarm[3] ".
                                        "$thisAlarm[4] $month[$thisAlarm[5]])";
                        } else {
                            $alarmMenuChoices[$n+2] = 
                                "Alarm $alarmNumber (".
                                    "$thisAlarm[2]:$thisAlarm[3] ".
                                        "$thisAlarm[4] $month[$thisAlarm[5]] - $thisAlarm[7])";
                        }
                    } else {
                        $alarmMenuChoices[$n+2] = 
                            "Alarm $alarmNumber (OFF)";
                    }
                }
            }
            
            
            my %params = (
                'listRef'        => \@alarmMenuChoices, 
                'externRef'		 => \@alarmMenuChoices, 
                'stringExternRef' => 1, 
                'externRefArgs'  => 'CV',
                'header'         => 'PLUGIN_SQUEEZED_VERA_ALARM', 
                'headerAddCount' => 1, 
                'stringHeader'   => 1, 
                'callback'       => \&squeezedveraAlarmMenu, 
                'onChange'       => \&squeezedveraOnChange, 
                'onChangeArgs'	 => 'CVI', 
            );
            
            # stuff that is provided by the system to scroll through the
            # options.
            Slim::Buttons::Common::pushModeLeft($client,
                                                'INPUT.List',
                                                \%params);
            $client->update;
            
        } elsif ($$valueref eq 'PLUGIN_SQUEEZED_VERA_SCENES') {

            $client->showBriefly( {
                'line'       => [ "", "Reading scenes..." ],
                'overlay'    => [ $client->symbols('notesymbol') ],
            }, { 'duration' => 1, 'block' => 1 } );
            
            # Find out which is the current scene
            my $sqvs_path = $prefs->get('sqvs_path');
            my $current_dir = getcwd;
           
            chdir $sqvs_path or die "chdir $sqvs_path: $!";
            my $current_scene = `./squeezedVeraScenes.pl -c`;
	    chdir $current_dir or die "chdir $current_dir: $!";
            
            $scenesMenuChoices[0] = "Current: $current_scene";
            $scenesMenuChoices[1] = 'PLUGIN_SQUEEZED_VERA_SCENES_CHANGE';
                        
            my %params = (
                'listRef'        => \@scenesMenuChoices, 
                'externRef'		 => \@scenesMenuChoices, 
                'stringExternRef' => 1, 
                'externRefArgs'  => 'CV',
                'header'         => 'PLUGIN_SQUEEZED_VERA_SCENES', 
                'headerAddCount' => 1, 
                'stringHeader'   => 1, 
                'callback'       => \&squeezedveraScenesMenu, 
                'onChange'       => \&squeezedveraOnChange, 
                'onChangeArgs'	 => 'CVI', 
            );
            
            # stuff that is provided by the system to scroll through the
            # options.
            Slim::Buttons::Common::pushModeLeft($client,
                                                'INPUT.List',
                                                \%params);
            $client->update;
            
        }
    } else
	{
            # just indicate nothing was done with a bump right
            $client->bumpRight();
            
            my $valueref = $client->modeParam('valueRef');
	}
}

sub squeezedveraScenesMenu {
    my ($client,$exittype) = @_;
    
    $exittype = uc($exittype);    

    if ($exittype eq 'LEFT') {
        Slim::Buttons::Common::popModeRight($client);
        $client->update;
    } elsif ($exittype eq 'RIGHT') {
        my $valueref = $client->modeParam('valueRef');

        if ( $$valueref eq 'PLUGIN_SQUEEZED_VERA_SCENES_CHANGE' ) {

            $client->showBriefly( {
                'line'       => [ "", "Reading scenes..." ],
                'overlay'    => [ $client->symbols('notesymbol') ],
            }, { 'duration' => 1, 'block' => 1 } );
            
            my $sqvs_path = $prefs->get('sqvs_path');
            my $current_dir = getcwd;
            
            chdir $sqvs_path or die "chdir $sqvs_path: $!";
            my $all_scenes = `./squeezedVeraScenes.pl -a`;
            chdir $current_dir or die "chdir $current_dir: $!";

            my @scenes = split / /, $all_scenes;
            my $numScenes = scalar @scenes;            
            my $n;
            for ( $n=0; $n<$numScenes; $n++ ) {
                $allScenesMenu[$n] = $scenes[$n];
            }

            my %params = (
                'listRef'        => \@allScenesMenu, 
                'externRef'		 => \@allScenesMenu, 
                'stringExternRef' => 1, 
                'externRefArgs'  => 'CV',
                'header'         => 'PLUGIN_SQUEEZED_VERA_SCENES', 
                'headerAddCount' => 1, 
                'stringHeader'   => 1, 
                'callback'       => \&allScenesMenu, 
                'onChange'       => \&squeezedveraOnChange, 
                'onChangeArgs'	 => 'CVI', 
            );
            
            # stuff that is provided by the system to scroll through the
            # options.
            Slim::Buttons::Common::pushModeLeft($client,
                                                'INPUT.List',
                                                \%params);
            $client->update;
            
        }
        
    } 
}

sub allScenesMenu {
    my ($client,$exittype) = @_;
    
    $exittype = uc($exittype);

    if ($exittype eq 'LEFT') {
        Slim::Buttons::Common::popModeRight($client);
        $client->update;
    } elsif ($exittype eq 'RIGHT') {

        $client->showBriefly( {
            'line'       => [ "", "Updating scenes..." ],
            'overlay'    => [ $client->symbols('notesymbol') ],
        }, { 'duration' => 1, 'block' => 1 } );
        
        my $pickedScene = $client->modeParam('valueRef');
        my $sqvs_path = $prefs->get('sqvs_path');
        my $current_dir = getcwd;
        
        chdir $sqvs_path or die "chdir $sqvs_path: $!";
        my $change_scene = `./squeezedVeraScenes.pl -n $$pickedScene`;
        chdir $current_dir or die "chdir $current_dir: $!";

        $newScene = $pickedScene;

        Slim::Buttons::Common::popModeRight($client);

        $scenesMenuChoices[0] = "Current: $$newScene";
        $scenesMenuChoices[1] = 'PLUGIN_SQUEEZED_VERA_SCENES_CHANGE';
        $client->update;

    }
}

my %iOldPowerState;
my %startedByAlarm;

# This sub routine is used at power on/off/play detection
sub commandCallback {
    my $request = shift;
    
    my $client = $request->client();
   
    my $pwr_on_scripts        = $prefs->get('pwr_on_scripts');
    my $pwr_off_scripts       = $prefs->get('pwr_off_scripts');
    my $play_scripts          = $prefs->get('play_scripts');
    my $alarm_timeout_scripts = $prefs->get('alarm_timeout_scripts');
    
    # Do nothing if client is not defined
    if(!defined($client)) {
        return;
    }

    # Extract the scripts to run
    my @pwrOnScripts        = split /;/, $pwr_on_scripts;
    my @pwrOffScripts       = split /;/, $pwr_off_scripts;
    my @playScripts         = split /;/, $play_scripts;
    my @alarmTimeoutScripts = split /;/, $alarm_timeout_scripts;
    
    #my $PowerSwitchEnabled = $prefs->client($client)->get( 'enabled');
    
    # Do nothing if power switch is disabled for this client
    #if( !defined( $PowerSwitchEnabled) || $PowerSwitchEnabled == 0) {
    #    &handlePowerOff( $client);
    #    return;
    #}
    
    
    # Get power on and off commands
    # Sometimes we do get only a power command, sometimes only a play/pause
    # command and sometimes both
    if( $request->isCommand([['power']])
            || $request->isCommand([['play']])
                || $request->isCommand([['pause']])
                    || $request->isCommand([['playlist'], ['newsong']])
                        || $request->isCommand([['stop']]) ) {

        my $iPower = $client->power();
        my $clientName = Slim::Player::Client::name($client);
        
        my $play_mode =  Slim::Player::Source::playmode($client);
        
        my $currentAlarm = Slim::Utils::Alarm->getCurrentAlarm($client);
        if ( defined $currentAlarm ) {
            $startedByAlarm{$client} = 1;
        }
        if ( !defined $startedByAlarm{$client} ) {
            $startedByAlarm{$client} = 0;
        }   

        # Check with last known power state -> if different switch output
        if( $iOldPowerState{$client} ne $iPower) {
            $iOldPowerState{$client} = $iPower;
            if( $iPower == 1) {
                foreach my $pwr_on_script (@pwrOnScripts) {
                    my @pwrOnScriptNames = split /,/, $pwr_on_script;
                    if ($clientName eq $pwrOnScriptNames[1]) {
                        system $script_path.$pwrOnScriptNames[0];
                        myDebug("$clientName, Power on, executing: $script_path.$pwrOnScriptNames[0]");
                        myDebug("$?");
                    }
                }
            } else {
                foreach my $pwr_off_script (@pwrOffScripts) {
                    my @pwrOffScriptNames = split /,/, $pwr_off_script ;
                    if ($clientName eq $pwrOffScriptNames[1]) {
                        $startedByAlarm{$client} = 0;
                        system $script_path.$pwrOffScriptNames[0];
                        myDebug("$clientName, Power off, executing: $script_path.$pwrOffScriptNames[0]");
                        myDebug("$?");
                    }
                }
            }
        }
        if ( $play_mode eq 'play' ) {
            foreach my $play_script (@playScripts) {
                my @playScriptNames = split /,/, $play_script;
                if ($clientName eq $playScriptNames[1]) {
                        system $script_path.$playScriptNames[0];
                        myDebug(
                            "$clientName, Play pressed, $playScriptNames[0]");
                    }
            }   
        }
        if( $request->isCommand([['pause']]) &&
                $startedByAlarm{$client} == 1 ) {
            foreach my $alarm_timeout_script (@alarmTimeoutScripts) {
                my @alarmTimeoutScriptNames = split /,/, $alarm_timeout_script;
                if ($clientName eq $alarmTimeoutScriptNames[1]) {
                    system $script_path.$alarmTimeoutScriptNames[0];
                    myDebug(
                        "$clientName, timedout, $alarmTimeoutScriptNames[0] ");
                }
                $startedByAlarm{$client} = 0;
            }
        }
        
    } elsif( $request->isCommand([['client'], ['new']])
                 || $request->isCommand([['client'], ['reconnect']])) {
        
        # Get current power state (needed for internal tracking)
        if( !defined( $iOldPowerState{$client})) {
            $iOldPowerState{$client} = $client->power();
        }
    } 
    
}


# Basically, any key press will pop you back to the previous menu.
sub doneShowingMsg {
    my ($client,$exittype) = @_;
      
    Slim::Buttons::Common::popModeRight($client);
}

# Everything is handled by the input modes stuff
# So, just return and empty hash - NOTE THE CURLY BRACES with the return call! 
sub getFunctions {
    return{};
}


# Always end with a 1 to make Perl happy
1;
